import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRoute } from '@angular/router';
import { ComponentFixture, fakeAsync, TestBed, tick } from '@angular/core/testing';

import { DataTableComponent } from './data-table.component';
import { DataTableColumn } from './data-table.model';
describe('DataTableComponent', () => {
  let component: DataTableComponent;
  let fixture: ComponentFixture<DataTableComponent>;

  let fetchRecordsEmitSpy: jasmine.Spy<any>;

  const mockDisplayedColumns: DataTableColumn[] = [
    {
      name: 'Fleet Name',
      field: 'name',
      width: '160px',
      sort: false,
      sortOrder: null,
    },
    {
      name: 'Description',
      field: 'description',
      width: '200px',
      sort: false,
      sortOrder: null,
    },
  ];

  const mockRecords = [
    { name: 'mock1', description: 'description1' },
    { name: 'mock2', description: 'description2' },
  ];

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [DataTableComponent],
      imports: [HttpClientTestingModule],
      providers: [
        {
          provide: ActivatedRoute,
          useValue: {
            snapshot: {
              data: { page: 'test-page' },
            },
          },
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(DataTableComponent);
    component = fixture.componentInstance;

    /* Assign @Input Bindings */
    component.columns = mockDisplayedColumns;
    component.records = mockRecords;

    fetchRecordsEmitSpy = spyOn(component.fetchRecords, 'emit');
    fixture.detectChanges();
  });

  it('should create the data table component', () => {
    expect(component).toBeTruthy();
  });

  it('should initialize the table', () => {
    /* Arrange */
    spyOn<any>(component, '_fetchRecords');
    /* Act */
    component.ngOnInit();
    /* Assert */
    expect(component.columns).toBe(mockDisplayedColumns);
    expect(component.records).toBe(mockRecords);
    expect(component['_fetchRecords']).toHaveBeenCalled();
    expect(component.fetchRecords.emit).toHaveBeenCalledWith({
      searchKey: '',
      paginationInfo: { pageNumber: 1, pageSize: 10 },
    });
  });
  it('should subscribe to search control changes', fakeAsync(() => {
    /* Arrange */
    const mockSearchKey = 'tes';
    spyOn<any>(component, '_resetPagination');
    spyOn<any>(component, '_fetchRecords').and.callThrough();
    fetchRecordsEmitSpy.calls.reset();
    fetchRecordsEmitSpy.and.callThrough();
    /* Act */
    component.searchControl.setValue(mockSearchKey);
    component.searchControl.updateValueAndValidity();
    /* Assert */
    expect(component.searchControl.value).toEqual(mockSearchKey);
    expect(component['_resetPagination']).not.toHaveBeenCalled();
    expect(component['_fetchRecords']).not.toHaveBeenCalled();
    tick(301);
    expect(component['_resetPagination']).toHaveBeenCalled();
    expect(component['_fetchRecords']).toHaveBeenCalled();
    expect(component.fetchRecords.emit).toHaveBeenCalledOnceWith({
      searchKey: mockSearchKey,
      paginationInfo: { pageNumber: 1, pageSize: 10 },
    });
  }));
  it('should update pagination based on API response data', () => {
    /* Arrange */
    spyOn<any>(component, '_setPaginationStartAndEnd').and.callThrough();
    /* Act */
    component.totalRecordsCount = 42;
    component.ngOnChanges({ records: {}, totalRecordsCount: { currentValue: 42 } } as any);
    /* Assert */
    expect(component['_setPaginationStartAndEnd']).toHaveBeenCalled();
    expect(component.currentPageStart).toBe(1);
    expect(component.currentPageEnd).toBe(10);

    /* Act */
    component.currentPageIndex = 5;
    component.ngOnChanges({ records: {}, totalRecordsCount: { currentValue: 42 } } as any);
    /* Assert */
    expect(component['_setPaginationStartAndEnd']).toHaveBeenCalled();
    expect(component.currentPageStart).toBe(41);
    expect(component.currentPageEnd).toBe(42);

    /* Act */
    (component as any)._resetPagination();
    /* Assert */
    expect(component.currentPageIndex).toBe(1);
    expect(component.currentPageStart).toBe(0);
    expect(component.currentPageEnd).toBe(0);

    /* Act */
    component.records = [];
    component.totalRecordsCount = 0;
    component.ngOnChanges({ records: { currentValue: [] }, totalRecordsCount: 0 } as any);
    /* Assert */
    expect(component['_setPaginationStartAndEnd']).toHaveBeenCalled();
    expect(component.currentPageStart).toBe(0);
    expect(component.currentPageEnd).toBe(0);
  });
  it('should change pagination current page based on user selection', () => {
    /* Arrange */
    spyOn<any>(component, '_fetchRecords').and.callThrough();
    fetchRecordsEmitSpy.calls.reset();
    fetchRecordsEmitSpy.and.callThrough();
    /* Act */
    component.onPageIndexChanged(7);
    /* Assert */
    component.currentPageIndex = 7;
    expect(component['_fetchRecords']).toHaveBeenCalled();
    expect(component.fetchRecords.emit).toHaveBeenCalledWith({
      searchKey: '',
      paginationInfo: { pageNumber: 7, pageSize: 10 },
    });
  });
  it('should change pagination page size based on user selection', () => {
    /* Arrange */
    spyOn<any>(component, '_fetchRecords').and.callThrough();
    fetchRecordsEmitSpy.calls.reset();
    fetchRecordsEmitSpy.and.callThrough();
    /* Act */
    component.currentPageIndex = 7;
    component.onPageSizeChanged(20);
    /* Assert */
    component.currentPageIndex = 1;
    component.pageSize = 20;
    expect(component['_fetchRecords']).toHaveBeenCalled();
    expect(component.fetchRecords.emit).toHaveBeenCalledWith({
      searchKey: '',
      paginationInfo: { pageNumber: 1, pageSize: 20 },
    });
  });
  it('should reset table', fakeAsync(() => {
    /* Arrange */
    component.searchControl.setValue('temp');
    component.searchControl.updateValueAndValidity();
    spyOn<any>(component, '_resetPagination').and.callThrough();
    spyOn<any>(component, '_fetchRecords').and.callThrough();
    spyOn(component, 'clearSearch').and.callThrough();
    fetchRecordsEmitSpy.calls.reset();
    fetchRecordsEmitSpy.and.callThrough();
    /* Act */
    component.reset();
    /* Assert */
    expect(component['_resetPagination']).toHaveBeenCalled();
    expect(component['clearSearch']).toHaveBeenCalledTimes(1);
    expect(component['_fetchRecords']).not.toHaveBeenCalled();
    expect(component.searchControl.value).toBe('');
    expect(component.fetchRecords.emit).not.toHaveBeenCalled();
    tick(300);
    expect(component.fetchRecords.emit).toHaveBeenCalledOnceWith({
      searchKey: '',
      paginationInfo: { pageNumber: 1, pageSize: 10 },
    });

    /* Act */
    fetchRecordsEmitSpy.calls.reset();
    fetchRecordsEmitSpy.and.callThrough();
    component.reset();
    /* Assert */
    expect(component['clearSearch']).toHaveBeenCalledTimes(1);
    expect(component['_fetchRecords']).toHaveBeenCalledTimes(2);
    expect(component.fetchRecords.emit).toHaveBeenCalledOnceWith({
      searchKey: '',
      paginationInfo: { pageNumber: 1, pageSize: 10 },
    });
  }));
});
