import { ComponentFixture, TestBed } from '@angular/core/testing';
import { By } from '@angular/platform-browser';
import { ActionButton } from '../models/common.model';

import { FooterComponent } from './footer.component';
import { UmbrellaFooterModule } from './umbrella-footer.module';

describe('FooterComponent', () => {
  let component: FooterComponent;
  let fixture: ComponentFixture<FooterComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [UmbrellaFooterModule],
      declarations: [FooterComponent],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(FooterComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render logo', () => {
    const logo = fixture.debugElement.query(By.css('.x-app-footer-logo'));
    expect(getComputedStyle(logo.nativeElement).backgroundImage).toContain('shell-recharge.svg');
  });

  it('should not display buttons by default', () => {
    const logo = fixture.debugElement.query(By.css('.x-app-footer-buttons'));
    expect(logo).toBeNull();
  });

  describe('primary button', () => {
    const mockPrimaryButton: ActionButton = { text: 'mock', action: () => {} };

    it('should be displayed conditionally', () => {
      component.primaryButton = mockPrimaryButton;
      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));
      expect(primaryButtonElement).toBeDefined();
      expect(primaryButtonElement.nativeElement.textContent.trim()).toEqual(mockPrimaryButton.text);
    });

    it('should be disabled conditionally', () => {
      mockPrimaryButton.disabled = true;
      component.primaryButton = mockPrimaryButton;
      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));
      expect(primaryButtonElement.attributes.disabled).toBe('true');
    });

    it('should display nz-icon conditionally', () => {
      mockPrimaryButton.nzIconName = 'ellipsis';
      component.primaryButton = mockPrimaryButton;
      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));
      const iconElement = primaryButtonElement.nativeNode.firstElementChild;
      expect(iconElement.className).toEqual('anticon anticon-ellipsis');
    });

    it('should execute calback action', () => {
      spyOn(mockPrimaryButton, 'action');
      component.primaryButton = mockPrimaryButton;
      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));
      primaryButtonElement.triggerEventHandler('click', null);
      expect(mockPrimaryButton.action).toHaveBeenCalled();
    });
  });

  describe('secondary buttons', () => {
    const firstSecondaryButtonMock: ActionButton = { text: 'mock sb 1', action: () => {} };
    const secondSecondaryButtonMock: ActionButton = { text: 'mock sb 2', action: () => {} };
    const mockSecondaryButtons: ActionButton[] = [
      firstSecondaryButtonMock,
      secondSecondaryButtonMock,
    ];

    it('should be displayed conditionally', () => {
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      expect(secondaryButtonsElements).toBeDefined();
      expect(secondaryButtonsElements.length).toBe(2);
      expect(secondaryButtonsElements[0].nativeElement.textContent.trim()).toBe(
        firstSecondaryButtonMock.text
      );
      expect(secondaryButtonsElements[1].nativeElement.textContent.trim()).toBe(
        secondSecondaryButtonMock.text
      );
    });

    it('should be disabled conditionally', () => {
      firstSecondaryButtonMock.disabled = true;
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      expect(secondaryButtonsElements[0].attributes.disabled).toBe('true');
      expect(secondaryButtonsElements[1].attributes.disabled).toBeUndefined();
    });

    it('should display nz-icon conditionally', () => {
      firstSecondaryButtonMock.nzIconName = 'ellipsis';
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      const iconElement = secondaryButtonsElements[0].nativeNode.firstElementChild;
      const buttonTextSpanElement = secondaryButtonsElements[1].nativeNode.firstElementChild;
      expect(iconElement.className).toEqual('anticon anticon-ellipsis');
      expect(buttonTextSpanElement.localName).toEqual('span');
    });

    it('should execute calback action', () => {
      spyOn(firstSecondaryButtonMock, 'action');
      spyOn(secondSecondaryButtonMock, 'action');
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      secondaryButtonsElements[0].triggerEventHandler('click', null);
      expect(firstSecondaryButtonMock.action).toHaveBeenCalled();
      expect(secondSecondaryButtonMock.action).not.toHaveBeenCalled();
    });
  });
});
