import { ComponentFixture, TestBed } from '@angular/core/testing';
import { By } from '@angular/platform-browser';
import { RouterTestingModule } from '@angular/router/testing';
import { ActionButton, MenuDropdownItem } from '../models/common.model';

import { HeaderComponent } from './header.component';
import { UmbrellaHeaderModule } from './umbrella-header.module';

describe('HeaderComponent', () => {
  let component: HeaderComponent;
  let fixture: ComponentFixture<HeaderComponent>;

  const mockTitle = 'mock title';
  const mockSubTitle = 'mock subTitle';
  const mockTags = [
    { text: 'tag 1', nzStatusColor: 'warning', color: 'blue' },
    { text: 'tag 2', color: 'blue' },
  ];
  const mockPrimaryButton: ActionButton = { text: 'mock', action: () => {} };
  const firstSecondaryButtonMock: ActionButton = { text: 'mock sb 1', action: () => {} };
  const secondSecondaryButtonMock: ActionButton = { text: 'mock sb 2', action: () => {} };
  const mockSecondaryButtons: ActionButton[] = [
    firstSecondaryButtonMock,
    secondSecondaryButtonMock,
  ];

  const firstEllipsisDropdownItem: MenuDropdownItem = { text: 'mock ei 1', action: () => {} };
  const secondEllipsisDropdownItem: MenuDropdownItem = { text: 'mock ei 2', action: () => {} };
  const mockEllipsisDropdownItems: MenuDropdownItem[] = [
    firstEllipsisDropdownItem,
    secondEllipsisDropdownItem,
  ];
  const mockBreadcrumbItems = [{ name: 'mock', routerLink: [] }];
  const mockNavRoutes = [{ name: 'mock', path: '' }];

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [UmbrellaHeaderModule, RouterTestingModule],
      declarations: [HeaderComponent],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(HeaderComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render background pattern image', () => {
    const patternSrsLogo = fixture.debugElement.query(By.css('.x-app-header'));
    expect(getComputedStyle(patternSrsLogo.nativeElement, '::before').backgroundImage).toContain(
      'pattern.svg'
    );
  });

  it('should display only required title by default', () => {
    component.title = mockTitle;

    fixture.detectChanges();
    const title = fixture.debugElement.query(By.css('nz-page-header-title'));
    const subTitle = fixture.debugElement.query(By.css('nz-page-header-subtitle'));
    const tags = fixture.debugElement.query(By.css('nz-page-header-tags'));
    const buttons = fixture.debugElement.query(By.css('button'));
    const ellipsisDropdownItems = fixture.debugElement.query(By.css('nz-dropdown-menu'));
    const breadcrumbItems = fixture.debugElement.query(By.css('nz-breadcrumb'));
    const navRoutes = fixture.debugElement.query(By.css('nz-tabset'));

    expect(title).toBeDefined();
    expect(title.nativeElement.textContent).toBe(mockTitle);
    expect(subTitle).toBeNull();
    expect(tags).toBeNull();
    expect(buttons).toBeNull();
    expect(ellipsisDropdownItems).toBeNull();
    expect(breadcrumbItems).toBeNull();
    expect(navRoutes).toBeNull();
  });

  it('should display subtitle conditionally', () => {
    component.subTitle = mockSubTitle;

    fixture.detectChanges();
    const subTitle = fixture.debugElement.query(By.css('nz-page-header-subtitle'));

    expect(subTitle).toBeDefined();
    expect(subTitle.nativeElement.textContent.trim()).toBe(mockSubTitle);
  });

  it('should display ellipsis dropdown conditionally', () => {
    component.ellipsisDropdownItems = mockEllipsisDropdownItems;

    fixture.detectChanges();
    const ellipsisDropdownButton = fixture.debugElement.query(By.css('.ant-dropdown-trigger'));
    const ellipsisDropdownMenu = fixture.debugElement.query(By.css('nz-dropdown-menu'));

    expect(ellipsisDropdownButton).toBeDefined();
    expect(ellipsisDropdownMenu).toBeDefined();
  });

  it('should display breadcrumbs conditionally', () => {
    component.breadcrumbItems = mockBreadcrumbItems;

    fixture.detectChanges();
    const breadcrumbItems = fixture.debugElement.query(By.css('nz-breadcrumb'));

    expect(breadcrumbItems).toBeDefined();
  });

  it('should display nav-tabs conditionally', () => {
    component.navRoutes = mockNavRoutes;

    fixture.detectChanges();
    const navRoutes = fixture.debugElement.query(By.css('nz-tabset'));

    expect(navRoutes).toBeDefined();
  });

  describe('tags', () => {
    it('should be displayed conditionally', () => {
      component.tags = mockTags;

      fixture.detectChanges();
      const tags = fixture.debugElement.query(By.css('nz-page-header-tags'));

      expect(tags).toBeDefined();
      expect(tags.nativeElement.firstElementChild.textContent.trim()).toBe(mockTags[0].text);
      expect(tags.nativeElement.lastElementChild.textContent.trim()).toBe(mockTags[1].text);
    });

    it('should prioritize nzStatusColor over color', () => {
      component.tags = mockTags;

      fixture.detectChanges();
      const tags = fixture.debugElement.query(By.css('nz-page-header-tags'));

      expect(tags).toBeDefined();

      expect(tags.nativeElement.firstElementChild.className).toContain('ant-tag-warning');
      expect(tags.nativeElement.firstElementChild.className).not.toContain('ant-tag-blue');
      expect(tags.nativeElement.lastElementChild.className).toContain('ant-tag-blue');
    });
  });

  describe('primary button', () => {
    it('should be displayed conditionally', () => {
      component.primaryButton = mockPrimaryButton;
      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));
      expect(primaryButtonElement).toBeDefined();
      expect(primaryButtonElement.nativeElement.textContent.trim()).toBe(mockPrimaryButton.text);
    });

    it('should be disabled conditionally', () => {
      mockPrimaryButton.disabled = true;
      component.primaryButton = mockPrimaryButton;
      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));
      expect(primaryButtonElement.attributes.disabled).toBe('true');
    });

    it('should display nz-icon conditionally', () => {
      mockPrimaryButton.nzIconName = 'ellipsis';
      component.primaryButton = mockPrimaryButton;

      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));

      const iconElement = primaryButtonElement.nativeNode.firstElementChild;
      expect(iconElement.className).toEqual('anticon anticon-ellipsis');
    });

    it('should execute calback action', () => {
      spyOn(mockPrimaryButton, 'action');
      component.primaryButton = mockPrimaryButton;
      fixture.detectChanges();
      const primaryButtonElement = fixture.debugElement.query(By.css('.ant-btn-primary'));
      primaryButtonElement.triggerEventHandler('click', null);
      expect(mockPrimaryButton.action).toHaveBeenCalled();
    });
  });

  describe('secondary buttons', () => {
    it('should be displayed conditionally', () => {
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      expect(secondaryButtonsElements).toBeDefined();
      expect(secondaryButtonsElements.length).toBe(2);
      expect(secondaryButtonsElements[0].nativeElement.textContent.trim()).toBe(
        firstSecondaryButtonMock.text
      );
      expect(secondaryButtonsElements[1].nativeElement.textContent.trim()).toBe(
        secondSecondaryButtonMock.text
      );
    });

    it('should be disabled conditionally', () => {
      firstSecondaryButtonMock.disabled = true;
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      expect(secondaryButtonsElements[0].attributes.disabled).toBe('true');
      expect(secondaryButtonsElements[1].attributes.disabled).toBeUndefined();
    });

    it('should display nz-icon conditionally', () => {
      firstSecondaryButtonMock.nzIconName = 'ellipsis';
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      const iconElement = secondaryButtonsElements[0].nativeNode.firstElementChild;
      const buttonTextSpanElement = secondaryButtonsElements[1].nativeNode.firstElementChild;
      expect(iconElement.className).toEqual('anticon anticon-ellipsis');
      expect(buttonTextSpanElement.localName).toEqual('span');
    });

    it('should execute calback action', () => {
      spyOn(firstSecondaryButtonMock, 'action');
      spyOn(secondSecondaryButtonMock, 'action');
      component.secondaryButtons = mockSecondaryButtons;
      fixture.detectChanges();
      const secondaryButtonsElements = fixture.debugElement.queryAll(By.css('.ant-btn'));
      secondaryButtonsElements[0].triggerEventHandler('click', null);
      expect(firstSecondaryButtonMock.action).toHaveBeenCalled();
      expect(secondSecondaryButtonMock.action).not.toHaveBeenCalled();
    });
  });
});
